#define SCR1
#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"
extern DBASE *db;

static gboolean sched_update(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  GtkWidget *window=Find_Parent_Window(widget);
  Update_Main_Wait(widget);
  refresh_wnd(window);
  return FALSE;
}


/*************************************
 * Update window (scheduled)
 *************************************/
void Update_Main(GtkWidget *widget)
{
  g_idle_add(sched_update,widget);
}

static RGBI rgb_bck;
void Update_Main_Wait(GtkWidget *widget)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *main_window=Find_Window(window,Lab_Main);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;

  db->redraw=TRUE;
  db->redrawing=TRUE;
  if (Get_Enable_Update(drawing_area))
  {
    db->redraw=FALSE;
    Set_Enable_Update(drawing_area,FALSE); /* Prevent re-drawing while drawing */
    draw_back(main_window);
    if (!db->redraw)
    {
      draw_rast(main_window,TRUE);
      Set_Adjust(main_window,SEC,"%d",db->glob_tm.tm_sec); /* force drawing and setting values */
    }
    else
    {
      db->redraw=FALSE;
      Update_Main(widget);
    }
    if (rgbi)
    {
      if (rgb_bck.rgbbuf) free(rgb_bck.rgbbuf);
      rgb_bck.bufsize=rgbi->bufsize;
      rgb_bck.rgbbuf=malloc(rgb_bck.bufsize);
      rgb_bck.width =rgbi->width;
      rgb_bck.height=rgbi->height;
      for (i=0; i<rgb_bck.bufsize; i++) *(rgb_bck.rgbbuf+i)=*(rgbi->rgbbuf+i);
    }
    Set_Enable_Update(drawing_area,TRUE);
  }
  db->redrawing=FALSE;
}

gint configure_event_main(GtkWidget         *widget,
                          GdkEventConfigure *event)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");

  Renew_RGBBuf(drawing_area);
  Update_Main(widget);
  return TRUE;
}

void zoom_inout(float *z,int *offset,float zfact,int w,char io)
{
  if (io=='i')
  {
    (*z)*=zfact;
    if (*z<1.) *z=1.;
    *offset=*offset+(int)(w/(*z)*(zfact-1.)/2.);
  }
  else
  {
    *offset=*offset-(int)(w/(*z)*(zfact-1.)/2.);
    (*z)/=zfact;
  }
}

void key_actions(GtkWidget *widget, GdkEventKey *event)
{
  GtkWidget *drawing_area=Find_Widget(widget,"GTK_DRAWING_AREA");
  GtkWidget *window=Find_Parent_Window(widget);
  int width=drawing_area->allocation.width/db->ppl;
  int height=drawing_area->allocation.height/db->ppl;
  gboolean redraw=FALSE;
#define FIX_ZMFACT 2.

  switch ((guchar)event->keyval)
  {
    case 'f':
      db->ox=0;
      db->oy=0;
      db->zx=db->zy=1.;
      redraw=TRUE;
    break;
    case 'I':
      zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'i');
      zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'i');
      redraw=TRUE;
    break;
    case 'i':
      db->zx*=FIX_ZMFACT;
      db->zy*=FIX_ZMFACT;
      db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
      db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
      redraw=TRUE;
    break;
    case 'O':
      zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'o');
      zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'o');
/* To prevent coredump in clear_wnd. rgb_bck.bufsize lacks rgb->bufsize! */
      rgb_bck.bufsize=0;
      redraw=TRUE;
    break;
    case 'o':
      db->zx/=FIX_ZMFACT;
      db->zy/=FIX_ZMFACT;
      db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
      db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);

/* To prevent coredump in clear_wnd. rgb_bck.bufsize lacks rgb->bufsize! */
      rgb_bck.bufsize=0;
      redraw=TRUE;
    break;
    case GDK_Left&0xff:
      db->ox-=(width/(db->zx*4));
      redraw=TRUE;
    break;
    case GDK_Right&0xff:
      db->ox+=(width/(db->zx*4));
      redraw=TRUE;
    break;
    case GDK_Up&0xff:
      db->oy-=(height/(db->zy*4));
      redraw=TRUE;
    break;
    case GDK_Down&0xff:
      db->oy+=(height/(db->zy*4));
      redraw=TRUE;
    break;
  }

  if (redraw)
  {
    db->setscrolflag=TRUE;
    Set_Scroll(window,"hbar",2*180,&db->ox,&db->zx,
                      "vbar",2* 90,&db->oy,&db->zy);
    db->setscrolflag=FALSE;
/*
    Update_Main(window);
    refresh_wnd(window);
*/
  }
}

void mouse_actions(GtkWidget *widget, GdkEventMotion *event)
{
  GdkModifierType state;
  int x,y;
  static gboolean button1_up;
  if (event->is_hint)
  {
    gdk_window_get_pointer(event->window,&x,&y,&state);
  }
  else
  {
    x=event->x;
    y=event->y;
    state=event->state;
  }
  if (state & GDK_BUTTON1_MASK)
  {
    if (!button1_up)
    {
      SAT *sat=nearest(db->sat,x,y);
      if (sat) Set_Entry(widget,LAB_SATN,"%s",sat->satname);
    }
    button1_up=TRUE;
  }
  if (!(state & GDK_BUTTON1_MASK))
  {
    button1_up=FALSE;
  }
}

void buttons_callback(GtkWidget *widget, gpointer data)
{
  GtkWidget *wnd=Find_Parent_Window(widget);
  char *name=(char *)data;
  static int to=0;

  if (!strcmp(name,RUN))
  {
    if ((!to) && (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
    {
      if (wnd)
        to=gtk_timeout_add(db->reptime*1000,draw_curpos,(gpointer)wnd);
    }
    else
    {
      if (to) gtk_timeout_remove(to); to=0;
    }
  }
  if (!strcmp(name,ASEL))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
      db->auto_select=TRUE;
    else
      db->auto_select=FALSE;
  }
  if (!strcmp(name,EXT))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
      db->ext_on=TRUE;
    else
      db->ext_on=FALSE;
  }

  if (!strcmp(name,SEL))
  {
    select_sat(wnd);
  }
  if (!strcmp(name,EKP))
  {
    Create_kepleredt_wnd(wnd);
  }
}

static void file_ok_sel(GtkWidget *widget, GtkFileSelection *fs)
{
  GtkWidget *file_window=Find_Parent_Window(widget);
  if (file_selection_get_filename())
  {
    strcpy(db->norad_file,(char *)file_selection_get_filename());
    strcpy(db->pref_noradfile,db->norad_file);
  }

  if (db->fp_norad) fclose(db->fp_norad);
  db->fp_norad=fopen(db->norad_file,"r");
  if (db->fp_norad)
  {
    Close_Fileselect(file_window); 
    fclose(db->fp_norad); db->fp_norad=NULL;
  }
  else
  {
    Create_Message("Error","Can't open file\n%s.\n",db->norad_file);
  }
}

void menu_callback(GtkWidget *widget, gpointer data)
{
  GtkWidget *wnd=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(widget,"GTK_DRAWING_AREA");
  int width=drawing_area->allocation.width/db->ppl;
  int height=drawing_area->allocation.height/db->ppl;

  char *name=(char *)data;
  gboolean redraw=FALSE;
  if (!strcmp(name,MENU_KF))
  {
    Create_Fileselectf(wnd,"Kepler-file",NULL,file_ok_sel,NULL,NULL,NULL,0,NULL);
  }

  if (!strcmp(name,MENU_SF))
  {
    select_sat(wnd);
  }
  if (!strcmp(name,MENU_EK))
  {
    Create_kepleredt_wnd(wnd);
  }
  if (!strcmp(name,MENU_PR))
  {
    Create_preferences_wnd(wnd);
  }
  if (!strcmp(name,MENU_PD))
  {
    Create_predict_wnd(wnd);
  }
  if (!strcmp(name,MENU_ZMFULL))
  {
    db->zx=1.;
    db->zy=1.;
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMIN))
  {
    db->zx*=FIX_ZMFACT;
    db->zy*=FIX_ZMFACT;
    db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
    db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMOUT))
  {
    db->zx/=FIX_ZMFACT;
    db->zy/=FIX_ZMFACT;
    db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
    db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMIN2))
  {
    zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'i');
    zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'i');
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMOUT2))
  {
    zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'o');
    zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'o');
    redraw=TRUE;
  }
  if (redraw)
  {
    Set_Scroll(wnd,"hbar",2*180 ,&db->ox,&db->zx,"vbar",2*90,&db->oy,&db->zy);
/*
    Update_Main(wnd);
*/
  }
}

#define SPINHOLSTOP
#ifdef SPINHOLSTOP
static gboolean idle_func(gpointer);
static guint idle_handler_id=0;


typedef struct widgetdata
{
  GtkWidget *w;
  gpointer d;
} WIDGETDATA;


void time_callback(GtkWidget *widget, gpointer data)
{
  static WIDGETDATA wd;
  wd.w=widget;
  wd.d=data;
  if (idle_handler_id==0)
  {
    idle_handler_id=g_idle_add(idle_func,&wd);
  }
}


static gboolean idle_func(gpointer data)
{
  WIDGETDATA *wd=(WIDGETDATA *)data;
  GtkWidget *widget=wd->w;
  char *name=(char *)wd->d;
#else
void time_callback(GtkWidget *widget, gpointer data)
{
  char *name=(char *)data;
#endif
  GtkWidget *wnd=Find_Parent_Window(widget);
  static gboolean dis=FALSE;

#ifdef SPINHOLSTOP
  idle_handler_id=0;
  if (db->redrawing) return FALSE;
  if (dis) return FALSE;
#else
  if (dis) return;
#endif
  dis=TRUE;

  if (!strcmp(name,SEC))
  {
    db->glob_tm.tm_sec=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,MINT))
  {
    db->glob_tm.tm_min=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,UUR))
  {
    db->glob_tm.tm_hour=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,DAG))
  {
    db->glob_tm.tm_mday=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,MND))
  {
    db->glob_tm.tm_mon=GTK_ADJUSTMENT(widget)->value-1;
  }
  if (!strcmp(name,JAAR))
  {
    db->glob_tm.tm_year=GTK_ADJUSTMENT(widget)->value-1900;
  }

  mktime_ntz(&db->glob_tm);

  Set_Adjust(wnd,SEC,"%d",db->glob_tm.tm_sec);
  Set_Adjust(wnd,MINT,"%d",db->glob_tm.tm_min);
  Set_Adjust(wnd,UUR,"%d",db->glob_tm.tm_hour);
  Set_Adjust(wnd,DAG,"%d",db->glob_tm.tm_mday);
  Set_Adjust(wnd,MND,"%d",db->glob_tm.tm_mon+1);
  Set_Adjust(wnd,JAAR,"%d",db->glob_tm.tm_year+1900);

  {
    GtkWidget *w;
    if (db->auto_select)
    {
      db->next_sat=get_next_sat(db->sat,&db->glob_tm);
    }

    if (!db->sat_sel) { dis=FALSE; return FALSE; }

    w=Find_Widget(wnd,db->sat_sel->satname);
    if (w) Set_Local_Button(w,SEL_LAB,TRUE);

    clear_wnd(wnd);
    draw_onepos_all(wnd);
    refresh_wnd(wnd);
  }
  dis=FALSE;

#ifdef SPINHOLSTOP
  return FALSE;
#else
  return;
#endif
}

void clear_wnd(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;
  for (i=0; i<rgb_bck.bufsize; i++) *(rgbi->rgbbuf+i)=*(rgb_bck.rgbbuf+i);
}
