#include <math.h>
#include <string.h>
#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"
#include "orbit.h"
#include "gif.h"


extern DBASE *db;

#define LENSIGN 10
FILE_TYPE detect_file(char *fn)
{

  char sign[LENSIGN];
#ifdef SUNSIGN
  guint32 *sign1=(guint32 *)sign;
#endif

  FILE_TYPE file_type=unknown_filetype;
  FILE *fp;

  if ((fp=fopen(fn,"rb")) == NULL) return unknown_filetype;
  fread(sign,4,1,fp);
  if (FALSE);
#ifdef SUNSIGN
  else if (GUINT32_FROM_BE(*sign1) == SUNSIGN)  file_type=SUN;
#endif
#ifdef GIFSIGN
  else if (!strncmp(sign,GIFSIGN,3)) file_type=GIF;
#endif
  fclose(fp);

  return file_type;
}

SAT *nearest(SAT *sat,int x,int y)
{
  SAT *sat1;
  SAT *satmin=NULL;
  gint32 dmin,d;
  int x1,y1;
  dmin=-1;
  for (sat1=sat; sat1; sat1=sat1->next)
  {
    sat1->selected=FALSE;
    x1=LON2X(R2D(sat1->pos.lon));
    y1=LAT2Y(R2D(sat1->pos.lat));
    d=(x1-x)*(x1-x)+(y1-y)*(y1-y);
    if ((dmin<0) || (d<dmin))
    {
      dmin=d;
      satmin=sat1;
    }
  }
  if (satmin) satmin->selected=TRUE;
  return satmin;
}

int draw_curpos(gpointer p)
{
  GtkWidget *wnd=(GtkWidget *)p;
  db->glob_tm=mom_tm();
  Set_Adjust(wnd,SEC,"%d",db->glob_tm.tm_sec);
  Set_Adjust(wnd,MINT,"%d",db->glob_tm.tm_min);
  Set_Adjust(wnd,UUR,"%d",db->glob_tm.tm_hour);
  Set_Adjust(wnd,DAG,"%d",db->glob_tm.tm_mday);
  Set_Adjust(wnd,MND,"%d",db->glob_tm.tm_mon+1);
  Set_Adjust(wnd,JAAR,"%d",db->glob_tm.tm_year+1900);
  return 1;
}

void draw_back(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr;
  FILE *fp;
  FILE_TYPE ftype;
  GIF_PICINFO *gif_pci;
  CLRMAP clrmap;
  int width,height;
  int picwidth,picheight;
  int picyp=-1;
  guint16 *line_r, *line_g, *line_b;
  int x,y;
  if (!drawing_area) return;
  if (!rgbi) return;
  gif_pci=calloc(sizeof(GIF_PICINFO),1);
  width=drawing_area->allocation.width;
  height=drawing_area->allocation.height;

  ftype=detect_file(db->map_file);
  if (!(fp=fopen(db->map_file,"rb"))) return;

  switch(ftype)
  {
#ifdef TIFFSIGN
    case TIFF:
      read_tiff_header(fp,&picwidth,&picheight);
    break;
#endif
#ifdef GIFSIGN
    case GIF:
      if (read_gif_header1(fp, &clrmap, gif_pci)) 
      {
        free(gif_pci);
        return;
      }
      picwidth=gif_pci->width;
      picheight=gif_pci->height;
    break;
#endif
    default:
      printf("Unknown filetype of %s\n",db->map_file);
      return;
    break;
  }

  line_r=malloc(3*picwidth*sizeof(guint16));
  line_g=malloc(3*picwidth*sizeof(guint16));
  line_b=malloc(3*picwidth*sizeof(guint16));

  for (y=0; y<LAT2Y(-90)-LAT2Y(90); y++)
  {
    int picy=(picheight*y)/(LAT2Y(-90)-LAT2Y(90));
    if (y+LAT2Y(90)>=height-RAND) continue;
    if (y+LAT2Y(90)<RAND) continue;
    g_main_iteration(FALSE);            /* Handle GTK actions now and then */
    if (y+LAT2Y(90)>=height-RAND) continue;
    if (y+LAT2Y(90)<RAND) continue;
    if (db->redraw) break;
    if (picy!=picyp)
    {
      switch(ftype)
      {
#ifdef TIFFSIGN
        case TIFF:
          read_tiff_line(fp,picwidth, picy,line_r,line_g,line_b);
        break;
#endif
#ifdef GIFSIGN
        case GIF:
          read_gif_line1(fp,gif_pci,picy,line_r);
        break;
#endif
        default: break;
      }
    }
    picyp=picy;

    for (x=0; x<=LON2X(180)-LON2X(-180); x++)
    {
      int picx=(picwidth*x)/(LON2X(180)-LON2X(-180));
      if (x+LON2X(-180)>=width-RAND) continue;
      if (x+LON2X(-180)<RAND) continue;
      switch(ftype)
      {
        case TIFF:
          clr.red=line_r[picx];
          clr.green=line_g[picx];
          clr.blue=line_b[picx];
        break;
        case GIF:
        {
          int n=*(line_r+picx);
          n=MIN(MAX(n,0),255);
          if (n<0) n=0;
          clr.red=clrmap.red[n];
          clr.green=clrmap.green[n];
          clr.blue=clrmap.blue[n];
        }
        break;
      default: break;
      }
      draw_rgbpoint(rgbi,&clr,x+LON2X(-180),y+LAT2Y(90));
    }
    Refresh_Rect(drawing_area,0,y+LAT2Y(90),drawing_area->allocation.width-1,1);

  }
  free(gif_pci);
  free(line_r);
  free(line_g);
  free(line_b);
  fclose(fp);
}

#define W_CHAR 8
void draw_rast(GtkWidget *wnd,gboolean rgbm)
{
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;
  int lon,lat;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor grey,white,num;
  grey.red=grey.green=grey.blue=0xa0;
  white.red=white.green=white.blue=0xf0;
  num.red=num.green=0x00; num.blue=0x00;

  if (rgbm)
  {
    char tmp[10];
    for (lon=-180; lon<=180; lon+=15)
    {
      draw_rgbline(rgbi,&grey,LON2X(lon),LAT2Y(-90),LON2X(lon),LAT2Y(90));
      sprintf(tmp,"%4d",lon);
      if ((ABS(lon)<180) && (!(lon%30)))
        draw_rgbstring(rgbi,&num,LON2X(lon)-2*W_CHAR,4,tmp);
    }
    draw_rgbline(rgbi,&white,LON2X(0),LAT2Y(-90),LON2X(0),LAT2Y(90));

    for (lat=-90; lat<=90; lat+=15)
    {
      draw_rgbline(rgbi,&grey,LON2X(-180),LAT2Y(lat),LON2X(180),LAT2Y(lat));
      sprintf(tmp,"%3d",lat);
      if ((ABS(lat)<90) && (!(lat%30)))
        draw_rgbstring(rgbi,&num,4,LAT2Y(lat)-W_CHAR,tmp);
    }
    draw_rgbline(rgbi,&white,LON2X(-180),LAT2Y(0),LON2X(180),LAT2Y(0));
  }
  else
  {
    for (lon=-180; lon<=180; lon+=15)
      gdk_draw_line(px,gc[RASCOL],LON2X(lon),LAT2Y(-90),LON2X(lon),LAT2Y(90));
    gdk_draw_line(px,gc[RASCOL2],LON2X(0),LAT2Y(-90),LON2X(0),LAT2Y(90));

    for (lat=-90; lat<=90; lat+=15)
      gdk_draw_line(px,gc[RASCOL],LON2X(-180),LAT2Y(lat),LON2X(180),LAT2Y(lat));
    gdk_draw_line(px,gc[RASCOL2],LON2X(-180),LAT2Y(0),LON2X(180),LAT2Y(0));
  }
}

#define SIZEREF 8
void draw_ref(GtkWidget *wnd,float lon,float lat,gboolean rgbm)
{
  int ilon,ilat;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;
  int x1,y1,x2,y2;
  ilon=R2D(lon);
  ilat=R2D(lat);
  x1=LON2X(ilon)-SIZEREF/2;
  x2=LON2X(ilon)+SIZEREF/2;
  y1=LAT2Y(ilat)-SIZEREF/2;
  y2=LAT2Y(ilat)+SIZEREF/2;
  if (rgbm)
  {
    clr1.red=clr1.blue=0xff; clr1.green=0;
    draw_rgbline(rgbi,&clr1,x1,y1,x1,y2);
    draw_rgbline(rgbi,&clr1,x1,y2,x2,y2);
    draw_rgbline(rgbi,&clr1,x2,y2,x2,y1);
    draw_rgbline(rgbi,&clr1,x2,y1,x1,y1);
  }
  else
  {
    gdk_draw_arc(px,gc[REFCOL],FALSE,x1,y1,SIZEREF,SIZEREF,0,360*64);
  }
}

#define SIZESAT 8
void draw_sat(GtkWidget *wnd,SAT *sat,float lon,float lat,gboolean rgbm)
{
  int x1,y1,x2,y2;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;

  lon=R2D(lon);
  lat=R2D(lat);
  x1=LON2X(lon)-SIZESAT/2;
  x2=LON2X(lon)+SIZESAT/2;
  y1=LAT2Y(lat)-SIZESAT/2;
  y2=LAT2Y(lat)+SIZESAT/2;
  if (rgbm)
  {
    clr1.red=0xff; clr1.green=0x00; clr1.blue=0x00;
    draw_rgbline(rgbi,&clr1,x1,y1,x1,y2);
    draw_rgbline(rgbi,&clr1,x1,y2,x2,y2);
    draw_rgbline(rgbi,&clr1,x2,y2,x2,y1);
    draw_rgbline(rgbi,&clr1,x2,y1,x1,y1);
    draw_rgbstring(rgbi,&clr1,x1+2*W_CHAR,y1+1*W_CHAR,sat->satname);
  }
  else
  {
    gdk_draw_arc(px,gc[SATCOL],FALSE,x1,y1,SIZESAT,SIZESAT,0,360*64);
  }
}

#define SIZE_MARK 8
void draw_mark(GtkWidget *wnd,POINT p,char *t)
{
  int x1,y1,x2,y2;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  float lon,lat;

  lon=R2D(p.lon);
  lat=R2D(p.lat);
  x1=LON2X(lon)-SIZE_MARK/2;
  x2=LON2X(lon)+SIZE_MARK/2;
  y1=LAT2Y(lat)-SIZE_MARK/2;
  y2=LAT2Y(lat)+SIZE_MARK/2;
  clr1.red=0xff; clr1.green=0x00; clr1.blue=0x00;
  draw_rgbline(rgbi,&clr1,x1,y1,x2,y2);
  draw_rgbline(rgbi,&clr1,x1,y2,x2,y1);
  draw_rgbstring(rgbi,&clr1,x1+2*W_CHAR,y1+1*W_CHAR,t);
}

void pijl(GdkPixmap *px,GdkGC *gc,int x,int y,int r)
{
  int dx1,dy1,dx2,dy2;

  dx1=10*cos(D2R(r-30));
  dy1=10*sin(D2R(r-30));
  dx2=10*cos(D2R(r+30));
  dy2=10*sin(D2R(r+30));
  gdk_draw_line(px,gc,x+dx1,y+dy1,x,y);
  gdk_draw_line(px,gc,x+dx2,y+dy2,x,y);
}

void pijlrgb(RGBI *rgbi,GdkColor *clr,int x,int y,int r)
{
  int dx1,dy1,dx2,dy2;

  dx1=10*cos(D2R(r-30));
  dy1=10*sin(D2R(r-30));
  dx2=10*cos(D2R(r+30));
  dy2=10*sin(D2R(r+30));
  draw_rgbline(rgbi,clr,x+dx1,y+dy1,x,y);
  draw_rgbline(rgbi,clr,x+dx2,y+dy2,x,y);
}

void draw_sattrack(GtkWidget *wnd,POINT p1,POINT p2,gboolean rgbm,int p)
{
  int x1,y1,x2,y2;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;

  x1=LON2X(R2D(p1.lon));
  x2=LON2X(R2D(p2.lon));
  y1=LAT2Y(R2D(p1.lat));
  y2=LAT2Y(R2D(p2.lat));

  if (ABS(x2-x1)>90) return;
  if (rgbm)
  {
    clr1.red=clr1.green=0x40; clr1.blue=0x40;
    draw_rgbline(rgbi,&clr1,x1,y1,x2,y2);
    if (p)
    {
      pijlrgb(rgbi,&clr1,x1,y1,(int)R2D(atan2(y2-y1,x2-x1)));
    }
  }
  else
  {
    gdk_draw_line(px,gc[TRACKCOL],x1,y1,x2,y2);
    if (p)
    {
      pijl(px,gc[TRACKCOL],x1,y1,(int)R2D(atan2(y2-y1,x2-x1)));
    }
  }
}

void draw_sensor(GtkWidget *wnd,SAT *sat,
                                 POINT *pos_sat,POINT *pos_earth,gboolean rgbm)
{
  POINT pos_subsat;
  int x;
  float lon,lat;
  float lonp,latp;
  POINT pos_sat2;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;
  if (!sat->orbit.max_sens_angle) return;

  for (x=0; x<2048; x++)
  {
    calc_x2satobs(&sat->kepler,&sat->orbit,
                  pos_sat,&pos_sat2,
                  sat->orbit.width_original,sat->orbit.max_sens_angle,
                  x);
    calcposrel(&sat->kepler,&sat->orbit,&pos_sat2,pos_earth,&pos_subsat);
    pos_subsat.lon-=D2R(sat->orbit.offset_lon);

    lon=R2D(pos_subsat.lon);
    lat=R2D(pos_subsat.lat);
    if (rgbm)
    {
      if (x<1024)
      {
        clr1.red=0x00; clr1.green=0x00; clr1.blue=0xff;
      }
      else
      {
        clr1.red=0x00; clr1.green=0x00; clr1.blue=0xff;
      }
      if ((x>0) && (ABS(lon-lonp) <30))
      {
        draw_rgbline(rgbi,&clr1,LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
        draw_rgbpoint(rgbi,&clr1,LON2X(lon),LAT2Y(lat));
    }
    else
    {
      int c;
      if (x<1024) c=SCANCOL; else c=SCANCOL2;
      if ((x>0) && (ABS(lon-lonp) <30))
      {
        gdk_draw_line(px,gc[c],LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
        gdk_draw_point(px,gc[c],LON2X(lon),LAT2Y(lat));
    }
    lonp=lon; latp=lat;
  }
}

void draw_vis(GtkWidget *wnd,SAT *sat,POINT *refpos,gboolean rgbm)
{
  float d;
  float e=0;
  float c;
  float lon,lat;
  float lonp=0,latp=0;
  gboolean first=TRUE;
  gboolean normal=TRUE;
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  GdkGC **gc=Get_Gc(wnd);
  GdkPixmap *px=Find_Widget(wnd,"GTK_DRAWING_AREA")->window;

  clr1.red=0xff; clr1.green=0; clr1.blue=0;
  d=acos(Rearth/(sat->orbit.height+Rearth)*cos(e))-e;
  for (c=0; c<=360; c+=1)
  {
    float cc=D2R(c);
    lat=asin(sin(refpos->lat)*cos(d) + cos(refpos->lat)*sin(d)*cos(cc));
    lon=asin(sin(cc)*sin(d)/cos(lat));

    if (((ABS(refpos->lat+d*cos(cc))) < PI/2.))
    {
      lon=R2D(lon);
      lat=R2D(lat);
      normal=TRUE;
    }
    else
    {
      if (lon>0) lon=180-R2D(lon); else lon=-180-R2D(lon);
      lat=R2D(lat);
      normal=FALSE;
    }

    lon+=R2D(refpos->lon);
    if (lon>180) lon-=360;
    if (lon<-180) lon+=360;
    if (rgbm)
    {
      if ((!first) && (ABS(lonp-lon) < 60))
      {
        draw_rgbline(rgbi,&clr1,LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
      {
        draw_rgbpoint(rgbi,&clr1,LON2X(lon),LAT2Y(lat));
      }
    }
    else
    {
      if ((!first) && (ABS(lonp-lon) < 60))
      {
        gdk_draw_line(px,gc[VISCOL],LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
      {
        gdk_draw_point(px,gc[VISCOL],LON2X(lon),LAT2Y(lat));
      }
    }
    first=FALSE;
    lonp=lon; latp=lat;
  }
}
