#include <math.h>
#include <stdlib.h>
#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"

extern DBASE *db;

void calcdraw_sensor(struct tm cur_tm,GtkWidget *wnd,SAT *sat)
{
  POINT pos_subsat;
  POINT pos_sat;
  POINT pos_earth;
  DIRECTION satdir;
  calc_onepos(sat,cur_tm,0,&pos_earth,&pos_sat,&pos_subsat);
  calceleazim(&pos_subsat,&sat->orbit,&db->refpos,&satdir);
  draw_sensor(wnd,db->sat_sel,&pos_sat,&pos_earth,TRUE);
}

void draw_track(GtkWidget *wnd,SAT *sat,struct tm tma,struct tm tmb)
{
  POINT pos_subsat1,pos_subsat2;
  POINT pos_sat;
  POINT pos_earth;
  int i=0;
  struct tm tm;
  for (tm=tma; mktime_ntz(&tm) < mktime_ntz(&tmb); tm.tm_min++)
  {
    calc_onepos(sat,tm,0,&pos_earth,&pos_sat,&pos_subsat1);
    if (i) draw_sattrack(wnd,pos_subsat1,pos_subsat2,TRUE,(i==1));
    pos_subsat2=pos_subsat1;
    i++;
  }
  calc_onepos(sat,tmb,0,&pos_earth,&pos_sat,&pos_subsat1);
  draw_sattrack(wnd,pos_subsat1,pos_subsat2,TRUE,TRUE);
}

void draw_trackpiece(GtkWidget *wnd,SAT *sat,struct tm cur_tm,gboolean rgbm)
{
  struct tm tma;
  struct tm tmb;
  int strtmin=-5;
  int stopmin=+40;

  tma=cur_tm; tma.tm_min+=strtmin;
  tmb=cur_tm; tmb.tm_min+=stopmin;
  draw_track(wnd,sat,tma,tmb);
}

void draw_onepos(struct tm cur_tm,GtkWidget *wnd,SAT *sat)
{
  POINT pos_subsat;
  POINT pos_sat;
  POINT pos_earth;
  DIRECTION satdir;
  gboolean rgb_flag=TRUE;

/*
  sat->orbit.width_original=2048;
  sat->orbit.max_sens_angle=D2R(55.4);
*/
  sat->orbit.scan_hor=E_W;

  if ((!sat->visible) && (!sat->selected)) return;

  if (sat->selected)
  {
    draw_trackpiece(wnd,sat,cur_tm,rgb_flag);
/*
    report_nextpass(wnd,sat);
*/
  }
  calc_onepos(sat,cur_tm,0,&pos_earth,&pos_sat,&pos_subsat);
  calceleazim(&pos_subsat,&sat->orbit,&db->refpos,&satdir);
  sat->pos.lon=pos_subsat.lon;
  sat->pos.lat=pos_subsat.lat;
  sat->dir.elev=satdir.elev;
  sat->dir.azim=satdir.azim;
  draw_sat(wnd,sat,pos_subsat.lon,pos_subsat.lat,rgb_flag);
  draw_sensor(wnd,sat,&pos_sat,&pos_earth,rgb_flag);
  draw_vis(wnd,sat,&pos_subsat,rgb_flag);

#define GOING_UP   ((prev_elev<db->elev_hor) && (satdir.elev>=db->elev_hor))
#define GOING_DOWN ((prev_elev>=db->elev_hor) && (satdir.elev<db->elev_hor))
  if (sat->selected)
  {
    char prog[100];
    static float prev_elev;
    Set_Entry(wnd,LON,"%.1f",R2D(pos_subsat.lon));
    Set_Entry(wnd,LAT,"%.1f",R2D(pos_subsat.lat));
    Set_Entry(wnd,ELE,"%.1f",R2D(satdir.elev));
    Set_Entry(wnd,AZI,"%.1f",R2D(satdir.azim));
    Set_Entry(wnd,XXX,"%.1f",R2D(satdir.x));
    Set_Entry(wnd,YYY,"%.1f",R2D(satdir.y));

    if (db->ext_on)
    {
      if (*db->prog_track)
      {
        strcpy(prog,db->prog_track);
        fillin_satpos(prog,sat->satname,R2D(satdir.elev),R2D(satdir.azim),R2D(satdir.x),R2D(satdir.y));
        system(prog);
      }
      if ((*db->prog_trackup) && (satdir.elev>=db->elev_hor))
      {
        strcpy(prog,db->prog_trackup);
        fillin_satpos(prog,sat->satname,R2D(satdir.elev),R2D(satdir.azim),R2D(satdir.x),R2D(satdir.y));
        system(prog);
      }
      if ((*db->prog_up) && (GOING_UP))
      {
        strcpy(prog,db->prog_up);
        fillin_satpos(prog,sat->satname,R2D(satdir.elev),R2D(satdir.azim),R2D(satdir.x),R2D(satdir.y));
        system(prog);
      }
      if ((*db->prog_down) && (GOING_DOWN))
      {
        strcpy(prog,db->prog_down);
        fillin_satpos(prog,sat->satname,R2D(satdir.elev),R2D(satdir.azim),R2D(satdir.x),R2D(satdir.y));
        system(prog);
      }
    }
    if (satdir.elev<db->elev_hor)
    {
      if (db->next_sat) db->sat_sel=db->next_sat;
      select_this_sat(db->sat_sel);
      report_nextpass(wnd,sat);
    }

    prev_elev=satdir.elev;
  }
}

void draw_onepos_all(GtkWidget *wnd)
{
  SAT *sat;
  if (db->redraw) return;
  for (sat=db->sat; sat; sat=sat->next)
  {
    draw_onepos(db->glob_tm,wnd,sat);
  }
}
