#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"

extern DBASE *db;
void ld_clr(GdkColor *clr,int i,int r,int g,int b)
{
  clr[i].red=r<<8;
  clr[i].green=g<<8;
  clr[i].blue=b<<8;
}

struct tm mom_tm()
{
  time_t t;
  struct tm tm;
  time(&t);
  tm=*gmtime(&t);
  tm.tm_hour+=(int)(db->time_offset);
  tm.tm_min+=(db->time_offset - (int)(db->time_offset))*60;
  mktime_ntz(&tm);
  return tm;
}

struct tm local2gmt(struct tm *ltm)
{
  time_t t;
  struct tm tm1,tm2;
  struct tm gmtm;
  time(&t);
  tm1=*gmtime(&t);
  tm2=*localtime(&t);
  gmtm=*ltm;
  gmtm.tm_hour+=(tm2.tm_hour-tm1.tm_hour);
  gmtm.tm_min+=(tm2.tm_min-tm1.tm_min);
  gmtm.tm_hour-=(int)(db->time_offset);
  gmtm.tm_min-=(db->time_offset - (int)(db->time_offset))*60;
  mktime_ntz(&gmtm);
  return gmtm;
}

void init_db(DBASE *db,char loc)
{
  switch(loc)
  {
    case 'E':
      db->refpos.lon=D2R(-2.);
      db->refpos.lat=D2R(52.);
    break;
    default:
      db->refpos.lon=D2R(5.);
      db->refpos.lat=D2R(52.);
    break;
  }
  db->elev_hor=0;
  db->ppl=FACT;
  db->zx=db->zy=1.;
  db->satsel_bottom=FALSE;
  db->glob_tm=mom_tm();
  db->genrange_tm.tm_mon=-1;
  db->genrange_tm.tm_mday=1;
  db->reptime=1;
  db->is_gmtime=TRUE;
}

/*************************************
 * translate days-in-year in tm struct into month and day
 *************************************/
void yday2mday_mon(struct tm *tmref)
{
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  if (tmref->tm_year/4==tmref->tm_year/4.) days[1]=29;

  tmref->tm_mday=tmref->tm_yday+1;
  tmref->tm_mon=0; 
  for (i=0; i<12; i++)
  {
    if (tmref->tm_mday<=days[i]) break;
    tmref->tm_mon++;
    tmref->tm_mday-=days[i];
  }

}

/*************************************
 * Correct tm-struct and return #seconds since 1970.
 * Similar to mktime, except that dayligt saving is ignored.
 *************************************/
gint32 mktime_ntz(struct tm *tm)
{
  gint32 secs;
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};

  while (tm->tm_sec >= 60) { tm->tm_min++; tm->tm_sec-=60; }
  while (tm->tm_sec <   0) { tm->tm_min--; tm->tm_sec+=60; }
  while (tm->tm_min >= 60) { tm->tm_hour++; tm->tm_min-=60; }
  while (tm->tm_min <   0) { tm->tm_hour--; tm->tm_min+=60; }
  while (tm->tm_hour>= 24) { tm->tm_mday++; tm->tm_hour-=24; }
  while (tm->tm_hour<   0) { tm->tm_mday--; tm->tm_hour+=24; }
  while (tm->tm_mon >= 12) { tm->tm_year++; tm->tm_mon-=12; }
  while (tm->tm_mon <   0) { tm->tm_year--; tm->tm_mon+=12; }
  if (tm->tm_year/4==tm->tm_year/4.) days[1]=29;

  while (tm->tm_mday>days[tm->tm_mon])
  {
    tm->tm_mday-=days[tm->tm_mon]; tm->tm_mon++;
    while (tm->tm_mon>= 12)  { tm->tm_year++; tm->tm_mon-=12; }
    if (tm->tm_year/4==tm->tm_year/4.) days[1]=29; else days[1]=28;
  }

  while (tm->tm_mday<=0)
  {
    tm->tm_mon--;
    tm->tm_mday+=days[tm->tm_mon];
    while (tm->tm_mon< 0)  { tm->tm_year--; tm->tm_mon+=12; }
    if (tm->tm_year/4==tm->tm_year/4.) days[1]=29; else days[1]=28;
  }

  secs=tm->tm_sec+60*(tm->tm_min+60*(tm->tm_hour+24*(tm->tm_mday-1)));
  for (i=0; i<tm->tm_mon; i++) secs+=(days[i]*24*3600);
  secs+=((tm->tm_year-70)*365*24*3600); 
  secs+=((tm->tm_year-70+1)/4)*24*3600; 
  tm->tm_yday=0;
  for (i=0; i<tm->tm_mon; i++) tm->tm_yday+=days[i];
  tm->tm_yday+=(tm->tm_mday-1);
  return secs;
}

char *timestr(struct tm *tm)
{
  static char s[50],*p;
  strcpy(s,asctime(tm));
  if ((p=strchr(s,'\n'))) *p=0;
  return s+4;
}

char *tmstrsh(struct tm *tm)
{
  static char s[51];
  strftime(s,50,"%d-%m %H:%M:%S",tm);
  return s;
}

char *tmstrshx(struct tm *tm,char *what)
{
  static char s[51];
  char frmt[20];
  strcpy(frmt,"");
  if (strchr(what,'m')) strcat(frmt,"%d-%m ");
  if (strchr(what,'H')) strcat(frmt,"%H");
  if (strchr(what,'M')) strcat(frmt,":%M");
  if (strchr(what,'S')) strcat(frmt,":%S");
  strftime(s,50,frmt,tm);
  return s;
}

void alloc_err(char *s)
{
  fprintf(stderr,"Mem allocate error!\n");
  if (s) fprintf(stderr,"%s\n",s);
  exit(1);
}

char *type_prcnt(char *p)
{
  while ((*p) && ((isdigit(*p)) || (*p=='.'))) p++;
  return p;
}


void fillin_satpos(char *frmt,char *name,float elev,float azim,float x,float y)
{
  char *p;
  char *rslt=malloc(strlen(frmt)+strlen(name)+50);
  for (p=frmt; *p; p++) if (*p=='%')
  {
    if (*(p+1)=='%')
    {
      p+=2; continue;
    }

    *p=1;
    
  }
  while ((p=strchr(frmt,1)))
  {
    *p='%';
    if ((p=type_prcnt(p+1)))
    {
      switch(*p)
      {
        case 'n': 
          *p='s';
          sprintf(rslt,frmt,name);
          strcpy(frmt,rslt);
        break;
        case 'e': 
          *p='f';
          sprintf(rslt,frmt,elev);
          strcpy(frmt,rslt);
        break;
        case 'a': 
          *p='f';
          sprintf(rslt,frmt,azim);
          strcpy(frmt,rslt);
        break;
        case 'x': 
          *p='f';
          sprintf(rslt,frmt,x);
          strcpy(frmt,rslt);
        break;
        case 'y': 
          *p='f';
          sprintf(rslt,frmt,y);
          strcpy(frmt,rslt);
        break;
        default:
        break;

      }
    }
  }
  free(rslt);
}

char *dbl_prct(char *p)
{
  static char tmp[1000];
  char *q=tmp;
  *tmp=0;
  for (; *p; p++)
  {
    if (*p=='%') { *q='%'; q++; }
    *q=*p; q++; *q=0;
  }
  return tmp;
}

/* Detect if file exist */
int exist_file(char *fn)
{
  FILE *fp;
  if (!(*fn)) return 0;
  if (!(fp=fopen(fn,"r"))) return 0;
  fclose(fp);
  return 1;
}
