#include <string.h>
#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"
extern DBASE *db;

TRACK *Create_Track(TRACK *track)
{
  TRACK *track_nw;

  track_nw=calloc(1,sizeof(TRACK));
  if (!track_nw) alloc_err("Create_Track");

  if (track)
  {
    while (track->next) track=track->next;

    track->next=track_nw;
    track_nw->prev=track;
  }
  return track_nw;
}

void remove_tracks(TRACK *track)
{
  while (track)
  {
    if (track->next)
    {
      track=track->next;
      free(track->prev);
    }
    else
    {
      free(track);
      break;
    }
  }
}

/* Calc. position of satellite at time 'cur_tm' */
void calc_onepos(SAT *sat,struct tm cur_tm,int cur_ms,
                 POINT *pos_earth,POINT *pos_sat,POINT *pos_subsat)
{
  struct tm cur_gmtm;
  cur_gmtm=cur_tm;
/*
Moet anders!
  if (!db->is_gmtime) cur_gmtm=local2gmt(&cur_tm);
*/
  calcposearth(&cur_gmtm,cur_ms,pos_earth);
  calcpossat(&cur_gmtm,cur_ms,&sat->kepler,&sat->orbit,pos_sat);
  calc_satobs(&sat->kepler,&sat->orbit,pos_sat,pos_sat,0);
  calcposrel(&sat->kepler,&sat->orbit,pos_sat,pos_earth,pos_subsat);
}

/* Search for satellite passing horizon */
gboolean search_horizon(SAT *sat,int sec_incr,
                        SEARCH_MODE search_mode,
                        struct tm *start_tm,  /* start time -> calc. time */
                        POINT *pos_subsat,float *mom_elev)
{
  POINT pos_sat;
  POINT pos_earth;
  DIRECTION satdir;
  float prev_elev=-10;
  int i;
  gint32 stopsec=24*60*sec_incr;

  for (i=0; i<stopsec; i+=sec_incr)
  {
    mktime_ntz(start_tm);

/* calc. mom. position */
    calc_onepos(sat,*start_tm,0,&pos_earth,&pos_sat,pos_subsat);
    calceleazim(pos_subsat,&sat->orbit,&db->refpos,&satdir);
    if (mom_elev) *mom_elev=satdir.elev;

    switch(search_mode)
    {
      case search_up:
        if (satdir.elev>=db->elev_hor) return TRUE;
      break;

      case search_maxelev:
        if (mom_elev)
        {
          if (*mom_elev < prev_elev) return TRUE;
          prev_elev=*mom_elev;
        }
      case search_down: /* Also case search_maxelev! */
        if (satdir.elev<db->elev_hor) return TRUE;
      break;
      default: break;
    }

    start_tm->tm_sec+=sec_incr;
  }
  return FALSE;
}

/****************************************************************
  Predict sat. overkomst
  sat         predict for this sat.
  start_tm    start searching from this time; returns up-time
  stop_tm     if !=NULL this will return down-time 
  tr          gets info about sat. track while above horizon
 ****************************************************************/
gboolean predict(SAT *sat,struct tm *start_tm,struct tm *stop_tm,TRACK *tr)
{
/* Search 'down' */
  if (!search_horizon(sat,30,search_down,start_tm,&tr->up_pos,NULL))
    return FALSE;

/* ========== Search satellite Up ========== */
/* Seach 'Up' fast */
  if (!search_horizon(sat,60,search_up,start_tm,&tr->up_pos,NULL))
    return FALSE;

/* Go back 1 min and search 'Up' precise */
  start_tm->tm_min--;
  start_tm->tm_sec++;
  if (!search_horizon(sat,1,search_up,start_tm,&tr->up_pos,NULL))
    return FALSE;
  tr->up_time=*start_tm;

/* Stop if no 'stop_time' requested */
  if (!stop_tm) return TRUE;

/* ========== Search satellite max. elevation ========== */
/* Asume 'Up', search 'Max. elev' fast, and determine max. elevation */
  *stop_tm=*start_tm;
  if (!search_horizon(sat,60,search_maxelev,stop_tm,&tr->maxelev_pos,&tr->max_elev))
    return FALSE;

/* Go back 1 min and search 'Max. elev' precise */
  stop_tm->tm_min--;
  stop_tm->tm_sec++;
  if (!search_horizon(sat,1,search_maxelev,stop_tm,&tr->maxelev_pos,&tr->max_elev))
    return FALSE;

  tr->maxelev_time=*stop_tm;

/* ========== Search satellite Down ========== */
/* Asume 'Up', search 'Down' fast */
  if (!search_horizon(sat,60,search_down,stop_tm,&tr->down_pos,NULL))
    return FALSE;

/* Go back 1 min and search 'Down' precise */
  stop_tm->tm_min--;
  stop_tm->tm_sec++;
  if (!search_horizon(sat,1,search_down,stop_tm,&tr->down_pos,NULL))
    return FALSE;

  tr->down_time=*stop_tm;

/* Ready; both start and stop time determined. */
  return TRUE;
}

/****************************************************************
  Get next visible sat. 
  sat           linked list of sats.
  genstart_tm   start searching from this time
  genstop_tm    stop searching time 
  Return:       track of first visible sat
 ****************************************************************/
SAT *get_next_sat(SAT *sat,struct tm *start_tm)
{
  TRACK track;
  SAT *next_sat=NULL;
  struct tm next_tm=*start_tm;
  next_tm.tm_year++;

  for (; sat; sat=sat->next)
  {
    struct tm tm=*start_tm;

    if (!sat->visible) continue;
    predict(sat,&tm,NULL,&track);

    if (mktime_ntz(&tm) < mktime_ntz(&next_tm))
    {
      next_sat=sat;
      next_tm=tm;
    }
  }
  return next_sat;
}

void select_this_sat(SAT *sat)
{
  SAT *sat1;
  for (sat1=sat; sat1->prev; sat1=sat1->prev);
  for (; sat1; sat1=sat1->next)
  {
    sat1->selected=FALSE;
  }
  sat->selected=TRUE;
}

/****************************************************************
  Generate list of overkomsten
  sat           predict for this sat.
  genstart_tm   start searching from this time
  genstop_tm    stop searching time 
  Return:       linked list of tracks
 ****************************************************************/
TRACK *gen_track(SAT *sat,struct tm genstart_tm,struct tm genstop_tm)
{
  GtkWidget *progress_bar;
  TRACK *track=NULL;
  TRACK track1;
  struct tm start_tm;
  struct tm stop_tm;
  gint32 predtimemax;
  gint32 predtime;

  start_tm=genstart_tm;
  predtimemax=mktime_ntz(&genstop_tm)-mktime_ntz(&start_tm);
  predtime=predtimemax;

  progress_bar=Create_Progress(NULL,"Predicting...");
  while (predtime > 0)
  {
    if (Update_Progress(progress_bar,predtimemax-predtime,predtimemax))
    {
      break;
    }
    if (predict(sat,&start_tm,&stop_tm,&track1))
    {
      if (mktime_ntz(&genstop_tm) > mktime_ntz(&start_tm))
      {
        if (track1.max_elev > db->elev_hor)
        {
          track=Create_Track(track);
          track1.prev=track->prev;
          track1.next=track->next;
          *track=track1;
        }
      }
    }
    predtime=mktime_ntz(&genstop_tm)-mktime_ntz(&start_tm);
  }
  Close_Window(gtk_widget_get_toplevel(progress_bar));
  Rewind(track);
  return track;
}


/****************************************************************
  Generate list of overkomsten and create list
  sat           predict for this sat.
  genstart_tm   start searching from this time
  genstop_tm    stop searching time 
 ****************************************************************/
void predict_range(GtkWidget *widget,SAT *sat,struct tm genstart_tm,struct tm genrange_tm)
{
  GtkWidget *w=Find_Widget(widget,"Updown");
  TRACK *track;

/* Create track */
  struct tm stop_tm=genstart_tm;
  stop_tm.tm_mday+=genrange_tm.tm_mday;
  stop_tm.tm_mon+=genrange_tm.tm_mon+1;
  stop_tm.tm_year+=genrange_tm.tm_year;
  mktime_ntz(&stop_tm);
  track=gen_track(sat,genstart_tm,stop_tm);

  add_to_list(w,track);
  remove_tracks(db->track);
  db->track=track;
}

void add_to_list(GtkWidget *w,TRACK *tr)
{
  char *tmp[4];
  char txt[4][100];
  gtk_clist_clear(GTK_CLIST(w));
  for (; tr; tr=tr->next)
  {
    sprintf(txt[0],"%s",tmstrsh(&tr->up_time));
    sprintf(txt[1],"%s",tmstrsh(&tr->down_time));
    sprintf(txt[2],"%s",tmstrsh(&tr->maxelev_time));
    sprintf(txt[3],"%4.1f",R2D(tr->max_elev));
    tmp[0]=txt[0];
    tmp[1]=txt[1];
    tmp[2]=txt[2];
    tmp[3]=txt[3];
    gtk_clist_append(GTK_CLIST(w), tmp);
  }
}

void pri_track(TRACK *tr)
{
  FILE *fp=stdout;
  for (; tr; tr=tr->next)
  {
    fprintf(fp,"Up  : (%7.2f,%7.2f) @ %s\n",
         R2D(tr->up_pos.lon),R2D(tr->up_pos.lat),timestr(&tr->up_time));
    fprintf(fp,"Down: (%7.2f,%7.2f) @ %s\n\n",
         R2D(tr->down_pos.lon),R2D(tr->down_pos.lat),timestr(&tr->down_time));
  }
}


void plot_startstop(FILE *fp,TRACK *track)
{
  int x,y,dx,dy;
  struct tm up_time=track->up_time;
  struct tm down_time=track->down_time;
  int clr;
  gboolean going_s=track->going_south;
  gboolean fill;
  gboolean in_color=FALSE;
  float max_elev=track->max_elev;
  going_s=(track->up_pos.lat>track->down_pos.lat? TRUE : FALSE);

  x=t2x(dagtijd(up_time));
  y=t2y(mndtijd(up_time));
  dx=t2x(dagtijd(down_time))-t2x(dagtijd(up_time));
  if (dx<0) dx=0;
  dy=3;

  if (in_color)
  {
    clr=(going_s? 0x0f0 : 0xf00);
    fill=TRUE;
  }
  else
  {
    clr=0x000;
    fill=going_s;
  }
  draw_rect(fp,x,y+1,dx,dy,clr,fill);
  draw_text(fp,x+dx,y,"%4.0f",R2D(max_elev));
}

void generate_ps(TRACK *track,SAT *sat)
{
  FILE *fp=stdout;
  char fn[100],*p;
  TRACK *track_mom;
  int min_hour=0,max_hour=24;
  int maxmon;
  int n=0;

/* Determine filename */
  sprintf(fn,"%s_%d",db->sat_sel->satname,db->genstart_tm.tm_mon+1);
  while ((p=strchr(fn,'/'))) *p='_';
  while ((p=strchr(fn,'-'))) *p='_';
  while ((p=strchr(fn,' '))) *p='_';
  strcat(fn,".ps");

  fp=fopen(fn,"w");
  if (!fp) { printf("??? Can't open %s!!!\n",fn); return; }

  db->ps_x_offset=min_hour;
  db->ps_y_offset=480;
  db->ps_max_h=200;
  db->ps_max_t=(max_hour-min_hour)*60;

  for (track_mom=track; track_mom; track_mom=track_mom->next)
  {
  }

  ps_hdr(fp);

  for (track_mom=track; track_mom; track_mom=track_mom->next)
  {
    if ((!track_mom->prev) ||
        (track_mom->prev->up_time.tm_mon != track_mom->up_time.tm_mon))
    {
      db->ps_y_offset-=(db->ps_max_h+40);
      maxmon=titel(fp,db,n++);
      ps_rast(fp,min_hour,max_hour,maxmon);
      setfont(fp,7);
    }
    plot_startstop(fp,track_mom);
  }
  ps_ldr(fp);

  if ((fp) && (fp!=stdout)) fclose(fp);
  Create_Message("Generated %s",fn);
}
