#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "xtrack.h"
#include "xtrack_func.h"

#define PREF_FILE ".xtrackrc"

#define SEPCHAR " :\n	"

char *get_fullarg(char *w)
{
  char *p;
  if (*w=='"')
  {
    w++;
    if ((p=strchr(w,'"')))
    {
      *p=0;
    }
    else
    {
      p=strtok(NULL,"\"");
      if (p) sprintf(w,"%s %s",w,p);
    }
  }
  return w;
}

void get_dirs(char *progname,char **cur_dir,char **prog_dir,char **home_dir)
{
  if (*cur_dir)  free(*cur_dir);  *cur_dir=NULL;
  if (*prog_dir) free(*prog_dir); *prog_dir=NULL;
  if (*home_dir) free(*home_dir); *home_dir=NULL;

  *cur_dir=g_get_current_dir();
  *prog_dir=get_path(progname);
  *home_dir=g_get_home_dir();
  #if __GTK_WIN32__ == 1
  {
    if (!(*home_dir))
    {
      if (!strncmp(*cur_dir+1,":\\",2))
      {
        *home_dir=malloc(10);
        strncpy(*home_dir,*cur_dir,2);
        *((*home_dir)+2)=0;
      }
    }
  }
  #endif
}

void search_file(char *rootfile,char *file,
                 char *cur_dir,char *home_dir,char *prog_dir)
{
/* Search pref-file */
  if (cur_dir)
    sprintf(file,"%s%c%s",cur_dir,DIR_SEPARATOR,rootfile);

  if (!(exist_file(file)))
  {
    *file=0;
    if (home_dir)
      sprintf(file,"%s%c%s",home_dir,DIR_SEPARATOR,rootfile);

    if (!(exist_file(file)))
    {
      *file=0;
      if (prog_dir)
        sprintf(file,"%s%c%s",prog_dir,DIR_SEPARATOR,rootfile);

      if (!(exist_file(file)))
      {
        *file=0;
      }
    }
  }
  return;
}

void Read_Prefs(char *progname,DBASE *db)
{
  FILE *fp;
  SAT *sat=NULL;
  char l[100],*w1,*w2,*w3,*w4;
  char satname[100];

/* Load directory names: current, program location etc. */
  get_dirs(progname,&db->cur_dir,&db->prog_dir,&db->home_dir);
  search_file(PREF_FILE,db->used_preffile,
              db->cur_dir,db->home_dir,db->prog_dir);
  if (!(fp=fopen(db->used_preffile,"r"))) return;
  while (fgets(l,100,fp))
  {
    w1=strtok(l,SEPCHAR);    if (!w1) continue;
    w2=strtok(NULL,SEPCHAR); if (!w2) continue;

    if ((!strcmp(w1,"Norad_file")) && (!*db->norad_file))
    {
      char sep[10];
      sprintf(sep,"%c.",DIR_SEPARATOR);
      strcpy(db->pref_noradfile,w2);

      #if __GTK_WIN32__ == 1
        if (*(db->pref_noradfile+1)==':')
          strcpy(db->norad_file,db->pref_noradfile);
        else
      #endif
        if (strchr(sep,*db->pref_noradfile))
          strcpy(db->norad_file,db->pref_noradfile);
        else
          search_file(db->pref_noradfile,db->norad_file,
                      db->cur_dir,db->home_dir,db->prog_dir);

    }
/*
    if (!strcmp(w1,"Hour_Offset"))  db->hour_offset=atoi(w2);
*/
    if (!strcmp(w1,"Hour_Offset"))  db->time_offset=atof(w2);
    if (!strcmp(w1,"Observer_Lon")) db->refpos.lon=D2R(atof(w2));
    if (!strcmp(w1,"Observer_Lat")) db->refpos.lat=D2R(atof(w2));
    if (!strcmp(w1,"Map_file"))
    {
      char sep[10];
      sprintf(sep,"%c.",DIR_SEPARATOR);
      strcpy(db->pref_mapfile,w2);

      #if __GTK_WIN32__ == 1
        if (*(db->pref_mapfile+1)==':')
          strcpy(db->map_file,db->pref_mapfile);
        else
      #endif
        if (strchr(sep,*db->pref_mapfile))
          strcpy(db->map_file,db->pref_mapfile);
        else
          search_file(db->pref_mapfile,db->map_file,
                      db->cur_dir,db->home_dir,db->prog_dir);
    }
    if (!strcmp(w1,"Rep_time"))     db->reptime=atoi(w2);
    if (!strcmp(w1,"Run_at_start"))
      db->start_now=(!strncmp(w2,"y",1)? TRUE : FALSE);
    if (!strcmp(w1,"Ext_progs"))
      db->ext_on=(!strncmp(w2,"y",1)? TRUE : FALSE);

    if (!strcmp(w1,"Prog_up"))      strcpy(db->prog_up,get_fullarg(w2));
    if (!strcmp(w1,"Prog_down"))    strcpy(db->prog_down,get_fullarg(w2));
    if (!strcmp(w1,"Prog_track"))   strcpy(db->prog_track,get_fullarg(w2));
    if (!strcmp(w1,"Prog_trackup")) strcpy(db->prog_trackup,get_fullarg(w2));

    if (!strcmp(w1,"Font_size"))    db->fontsize=atoi(w2);

    if (!strcmp(w1,"Buttonsatsel_position"))
      db->satsel_bottom=(!strcmp(w2,"bottom")? TRUE : FALSE);

    if ((!strcmp(w1,"Satellite")) && (!*db->satname))
    {
      SAT *sat1;
      w2=get_fullarg(w2);
      strcpy(satname,w2);
//puts(db->norad_file);
      if ((!(db->fp_norad=fopen(db->norad_file,"r"))))
      { 
        Create_Message("Warning2","Kepler-file '%s' not found.",db->norad_file);
        return;
      }

      sat1=read_msat(db->fp_norad,satname,db->sat);
      if (!sat)
        sat=sat1;
      else if (sat1)
      {
        SAT *s;
        for (s=sat; (s) && (s->next); s=s->next);
        s->next=sat1; sat1->prev=s;
        w3=strtok(NULL,SEPCHAR);
        w4=strtok(NULL,SEPCHAR);
        if ((w3) && (*w3=='1')) sat1->selected=TRUE;
        if ((w4) && (*w4=='1')) sat1->visible=TRUE;
      }
      if (db->fp_norad) fclose(db->fp_norad); db->fp_norad=NULL;
    }
  }
  if (sat)
  {
    add_to_db(db,sat,0);
    strcpy(db->satname," ");
  }
  fprintf(stderr,"Preference file %s found.\n",db->used_preffile);
}

void Save_Prefs(DBASE *db,gboolean save_sat)
{
  SAT *sat;
  FILE *fp=stdout;
  if (!(fp=fopen(PREF_FILE,"w")))
  {
    Create_Message("Error","Can't open pref-file %s for write.",PREF_FILE);
    return;
  }

  fprintf(fp,"Norad_file   : %s\n",db->pref_noradfile);
  fprintf(fp,"Hour_Offset  : %3.1f\n",db->time_offset);
/*
  fprintf(fp,"Hour_Offset  : %d\n",db->hour_offset);
*/
  fprintf(fp,"Observer_Lon : %f\n",R2D(db->refpos.lon));
  fprintf(fp,"Observer_Lat : %f\n",R2D(db->refpos.lat));
  fprintf(fp,"Map_file     : %s\n",db->pref_mapfile);
  fprintf(fp,"Rep_time     : %d\n",db->reptime);
  fprintf(fp,"Run_at_start : %s\n",(db->start_now? "y" : "n"));
  fprintf(fp,"Ext_progs    : %s\n",(db->ext_on? "y" : "n"));
  fprintf(fp,"Prog_up      : \"%s\"\n",db->prog_up);
  fprintf(fp,"Prog_down    : \"%s\"\n",db->prog_down);
  fprintf(fp,"Prog_track   : \"%s\"\n",db->prog_track);
  fprintf(fp,"Prog_trackup : \"%s\"\n",db->prog_trackup);

  fprintf(fp,"Font_size    : %d\n",db->fontsize);
  fprintf(fp,"Buttonsatsel_position: %s\n",
              (db->satsel_bottom?"bottom":"right"));

  if (save_sat)
  {
    for (sat=db->sat; sat; sat=sat->next)
    {
      fprintf(fp,"Satellite: \"%s\" %d %d\n",
                           sat->satname,sat->selected,sat->visible);
    }
  }
  fclose(fp);
}

extern DBASE *db;

#define LAB_SAVE "Save preferences"
#define LAB_CLOSE "Close"
#define LAB_LOPREF "Local preference file"
#define LAB_GLPREF "Used preference file"
#define LAB_ISGM     "Show time as GMT "
#define LAB_POSLON   "Longitude      "
#define LAB_POSLAT   "Latitude      "

#define LAB_PROGTRACK   "Run always          "
#define LAB_PROGUP      "Run 1x at sat. up   "
#define LAB_PROGDOWN    "Run 1x at sat. down "
#define LAB_PROGTRACKUP "Run during sat. up  "

#define LAB_KEPF "Kepler file"
#define LAB_MAPF "Map file   "

#define LAB_POSSATSEL   "Button position"
#define LAB_POSSATSEL_R "Right"
#define LAB_POSSATSEL_B "Bottom"
#define LAB_REPTIM      "Repetition time"
#define LAB_START       "Run at program start"
#define LAB_EXTPROG     "Execute external progs"
#define LAB_FONTS       "Font size"
#define LAB_OFFSETTM    "Offset gm time "

void callback_pref(GtkWidget *widget,gpointer data)
{
  char *name=(char *)data;
  static char *kfn,*mfn;
  if (!strcmp(name,LAB_OFFSETTM))
  {
    db->time_offset=GTK_ADJUSTMENT(widget)->value;
/*
    db->hour_offset=GTK_ADJUSTMENT(widget)->value;
*/
  }
  if (!strcmp(name,LAB_ISGM))
  {
    db->is_gmtime=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_POSLON))
  {
    db->refpos.lon=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_POSLAT))
  {
    db->refpos.lat=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_REPTIM))
  {
    db->reptime=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_PROGUP))
  {
    strcpy(db->prog_up,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGUP));
  }
  if (!strcmp(name,LAB_PROGDOWN))
  {
    strcpy(db->prog_down,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGDOWN));
  }
  if (!strcmp(name,LAB_PROGTRACK))
  {
    strcpy(db->prog_track,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGTRACK));
  }
  if (!strcmp(name,LAB_PROGTRACKUP))
  {
    strcpy(db->prog_trackup,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGTRACKUP));
  }

  if (!strcmp(name,LAB_KEPF))
  {
    kfn=Get_Entry(gtk_widget_get_toplevel(widget),LAB_KEPF);
  }
  if (!strcmp(name,LAB_MAPF))
  {
    mfn=Get_Entry(gtk_widget_get_toplevel(widget),LAB_MAPF);
  }

  if (!strcmp(name,LAB_FONTS))
  {
    db->fontsize=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_POSSATSEL_B))
    db->satsel_bottom=TRUE;
  if (!strcmp(name,LAB_POSSATSEL_R))
    db->satsel_bottom=FALSE;
  if (!strcmp(name,LAB_START))
  {
    db->start_now=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_EXTPROG))
  {
    db->ext_on=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_SAVE))
  {
    if (kfn) strcpy(db->pref_noradfile,kfn);
    if (mfn) strcpy(db->pref_mapfile,mfn);
    Save_Prefs(db,TRUE);
  }
  if (!strcmp(name,LAB_CLOSE))
  {
    GtkWidget *wnd=gtk_widget_get_toplevel(widget);
    kfn=NULL;
    mfn=NULL;
    Close_Window(wnd);
  }
}

void Create_preferences_wnd(GtkWidget *widget)
{
  GtkWidget *main_window=gtk_widget_get_toplevel(widget);
  GtkWidget *wnd,*wt[3],*wa,*wb,*wc,*wd1,*wd2,*wd,*we,*wx,*wy1,*wy2,*wy,*wz;
  char progs[4][100];
  strcpy(progs[0],dbl_prct(db->prog_track));
  strcpy(progs[1],dbl_prct(db->prog_up));
  strcpy(progs[2],dbl_prct(db->prog_down));
  strcpy(progs[3],dbl_prct(db->prog_trackup));

  if (!(wnd=Create_Window(main_window,0,0,"Preferences",NULL))) return;

  if (exist_file(PREF_FILE))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE); 
  else
    wt[1]=Create_Entry(LAB_LOPREF,NULL,"Not found."); 

  if (exist_file(db->used_preffile))
    wt[2]=Create_Entry(LAB_GLPREF,NULL,db->used_preffile); 
  else
    wt[2]=Create_Entry(LAB_GLPREF,NULL,"Not found"); 

  wt[0]=Pack("Preference file",'v',wt[1],5,wt[2],5,NULL);

  wa=Create_ButtonArray("Observator position",callback_pref,2,
    SPIN,LAB_POSLON  ,"%4f%f%f",R2D(db->refpos.lon),-180.,180.,
    SPIN,LAB_POSLAT  ,"%4f%f%f",R2D(db->refpos.lat),-90.,90.,
    SPIN,LAB_OFFSETTM,"%3.1f%f%f",db->time_offset,-23.,23.,
/*
    TOGGLE,LAB_ISGM,db->is_gmtime,
*/
                         0);

  wb=Create_ButtonArray("Files",callback_pref,1,
    ENTRY,LAB_KEPF,"%-20s",db->pref_noradfile,
    ENTRY,LAB_MAPF,"%-20s",db->pref_mapfile,
                         0);

  wc=Create_ButtonArray("Programs",callback_pref,1,
    ENTRY,LAB_PROGTRACK  ,"%-30s",progs[0],
    ENTRY,LAB_PROGUP     ,"%-30s",progs[1],
    ENTRY,LAB_PROGDOWN   ,"%-30s",progs[2],
    ENTRY,LAB_PROGTRACKUP,"%-30s",progs[3],
                         0);


  wd1=Create_ButtonArray("",callback_pref,1,
                        TOGGLE,LAB_START,db->start_now,
                        TOGGLE,LAB_EXTPROG,db->ext_on,
                        SPIN,LAB_REPTIM  ,"%3d%3d%3d",db->reptime,1,10000,
                        0);
  wd2=Create_Optionmenu(LAB_POSSATSEL,callback_pref,(db->satsel_bottom? 1 : 0),
                        LAB_POSSATSEL_R,LAB_POSSATSEL_B,
                        0);

  wd2=Pack(NULL,'v',wd2,5,NULL);
  wd=Pack("",'h',wd1,5,wd2,5,NULL);

  we=Create_Spin(LAB_FONTS,callback_pref,"%3d%3d%3d%d",10,db->fontsize,0,200);

/* The notebook */
  wx=Create_Notebook(NULL,GTK_POS_TOP,
                     "Observator position",wa,
                     "Files"              ,wb,
                     "Programs"           ,wc,
                     "Run options"        ,wd,
                     "Fonts & colors"     ,we,
                     NULL);

  wy1=Create_Button(LAB_SAVE,callback_pref);
  wy2=Create_Button(LAB_CLOSE,callback_pref);
  wy=SPack("","h",wy1,"fe5",wy2,"fe5",NULL);

  wz=Pack(NULL,'v',wt[0],5,wx,5,wy,5,NULL);

  gtk_container_add(GTK_CONTAINER(wnd),wz);
  gtk_widget_show(wnd);
  
}
