#define SCR1
#include <math.h>
#include <stdlib.h>
#include <time.h>

#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"
#include "orbit.h"

DBASE *db;

/* defaults */

#ifndef RELEASE
#define RELEASE "Unknown"
#endif

void help(char *p)
{
  fprintf(stderr,"%s release %s\n",p,RELEASE);
  fprintf(stderr,"%s [options]\n",p);
  fprintf(stderr,"   [options:]\n");
  fprintf(stderr,"     -f <norad-filename> (default: %s)\n",DEF_NORADFILE);
  fprintf(stderr,"     -s <sat_name> (default: %s)\n",DEF_SAT);
  fprintf(stderr,"     -rlon <ref-pos lon> (default: %d)\n",5);
  fprintf(stderr,"     -rlat <ref-pos lat> (default: %d)\n",52);
  fprintf(stderr,"     -rep_time <rep. time> (default: %d)\n",db->reptime);
  fprintf(stderr,"     -run_at_start\n");
  fprintf(stderr,"     -m <map-filename> (default: %s)\n",DEF_MAPFILE);
  fprintf(stderr,"     -ppl <value> (default: %2.0f)\n",FACT);
/*
  fprintf(stderr,"     -width <value> (default: %d)\n",720);
  fprintf(stderr,"     -rotordrv <program>\n");
*/
  fprintf(stderr,"  ppl=pixels per longitude.\n");
  fprintf(stderr,"  ppl=1 means: 1 pixel=1 degree, so size is 360x180 pixels.\n");
  fprintf(stderr,"  width=# pixels in x (lon) direction.\n");
}

GtkRcStyle *Create_RcStylex(GdkColor *normal,GdkColor *prelight,GdkColor *active)
{
  GtkRcStyle *rc_style;
  rc_style=gtk_rc_style_new();
  if (normal)
  {
    rc_style->bg[GTK_STATE_NORMAL]=*normal;
    rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BG;
  }
  if (prelight)
  {
    rc_style->bg[GTK_STATE_PRELIGHT]=*prelight;
    rc_style->color_flags[GTK_STATE_PRELIGHT] |= GTK_RC_BG;
  }
  if (active)
  {
    rc_style->bg[GTK_STATE_ACTIVE]=*active;
    rc_style->color_flags[GTK_STATE_ACTIVE] |= GTK_RC_BG;
  }
  return rc_style;
}
GtkWidget *Create_Toggle_c(char *name,        /* name of the button */
                          void func(),       /* function to be executed */
                          gboolean *act,
                          GdkColor *normal,
                          GdkColor *prelight,
                          GdkColor *active)
{
  GtkWidget *button;
  GtkRcStyle *rc_style;

  button=Create_Toggle(name,func,FALSE);
  rc_style=Create_RcStylex(normal,prelight,active);
  gtk_widget_modify_style(button,rc_style);


  return button;
}

#ifdef SCR1
void scroll_actions(GtkWidget *widget, gpointer data)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  char *name=(char *)data;
  float value,lower,upper;
  value=GTK_ADJUSTMENT(widget)->value;
  upper=GTK_ADJUSTMENT(widget)->upper;
  lower=GTK_ADJUSTMENT(widget)->lower;

  if (!db->setscrolflag)       /* To prevent conflict ox/ox ch. in Set_Scrol */
  {
    if (!strcmp(name,"hbar"))
    {
      db->ox=value-lower;
    }
    if (!strcmp(name,"vbar"))
    {
      db->oy=value-lower;
    }
  }
  Update_Main(widget);
  expose_event(drawing_area,NULL);
}
#endif

#define BKNOPPEN_H 200
#define RKNOPPEN_W 200

void Menu_Help(GtkWidget *widget,gpointer data)
{
#if __GTK_WIN32__ == 1
  Create_Message("Program info","xtrack release %s\n%c R. Alblas\nwerkgroep Kunstmanen\nNederland\n",RELEASE,'c');
#else
  Create_Message("Program info","xtrack release %s\n%c R. Alblas\nwerkgroep Kunstmanen\nNederland\n",RELEASE,GDK_copyright);
#endif
}

int main(int argc, char **argv)
{
  GtkWidget *main_window,*canvas,*menu;
  GdkColor clr[MAXCOL];
  GdkColormap clrmap;
  
  int i;
  int width=0;
  int height=0;
  int scrw;
  int scrh;

  int cvsw,cvsh;

  gboolean all_on=FALSE;
  gboolean sat_arg=FALSE;

  gtk_init(&argc, &argv);
  gdk_rgb_init();

  db=(DBASE *)calloc(1,sizeof(*db));
  if (!db) alloc_err("db");
  scrw=gdk_screen_width();
  scrh=gdk_screen_height();

/*
  Read_Prefs(argv[0],db);
  init_db(db,0);
  search_file(DEF_MAPFILE,db->map_file,
              db->cur_dir,db->home_dir,db->prog_dir);
*/
  for (i=1; i<argc; i++)
  {
    if ((!strcmp(argv[i-1],"-f"))) strcpy(db->norad_file,argv[i]);
/*
werkt nuet
    if ((!strcmp(argv[i-1],"-s")))
    {
      strcpy(db->satname,argv[i]);
      sat_arg=TRUE;
    }
*/
  }

  init_db(db,0);
  Read_Prefs(argv[0],db);
  search_file(DEF_MAPFILE,db->map_file,
              db->cur_dir,db->home_dir,db->prog_dir);

  db->fontsize=set_fontsize(db->fontsize);

  if (!*db->norad_file) strcpy(db->norad_file,DEF_NORADFILE);

  if (!*db->satname)
  {
    strcpy(db->satname,DEF_SAT);
/*    sat_arg=TRUE;*/
  }

  width=660+RKNOPPEN_W;
  height=(width-RKNOPPEN_W)*190/370+BKNOPPEN_H;
  for (i=1; i<argc; i++)
  {
    if ((!strcmp(argv[i],"-h")))              { help(argv[0]); return 0; }
    if ((!strcmp(argv[i-1],"-m")))            strcpy(db->map_file,argv[i]);
    if ((!strcmp(argv[i-1],"-rlon")))         db->refpos.lon=D2R(atof(argv[i]));
    if ((!strcmp(argv[i-1],"-rlat")))         db->refpos.lat=D2R(atof(argv[i]));
    if ((!strcmp(argv[i],"-all_on")))         all_on=TRUE;
    if ((!strcmp(argv[i-1],"-rep_time")))     db->reptime=atof(argv[i]);
    if ((!strcmp(argv[i],"-run_at_start")))   db->start_now=TRUE;
    if ((!strcmp(argv[i-1],"-ppl")))          db->ppl=atof(argv[i]);
    if ((!strcmp(argv[i-1],"-offset")))       db->time_offset=atof(argv[i]);
    if ((!strcmp(argv[i-1],"-prog_up")))      strcpy(db->prog_up,argv[i]);
    if ((!strcmp(argv[i-1],"-prog_down")))    strcpy(db->prog_down,argv[i]);
    if ((!strcmp(argv[i-1],"-prog_track")))   strcpy(db->prog_track,argv[i]);
    if ((!strcmp(argv[i-1],"-prog_trackup"))) strcpy(db->prog_trackup,argv[i]);
    if ((!strcmp(argv[i-1],"-width")))
    {
      width=atof(argv[i])+RKNOPPEN_W;
      height=(width-RKNOPPEN_W)*190/370+BKNOPPEN_H;
    }
  }
  db->fact=db->ppl;

/* Clip fact: 0.1 ... 8 */
  db->fact=MAX(MIN(db->fact,8),0.1);

  if ((width<0) && (370*db->fact > scrw))
  {
    width=scrw; height=width*190/370+BKNOPPEN_H;
  }

  cvsw=(360+RAND)*db->fact;
  cvsh=(180+RAND)*db->fact;

  if (sat_arg)
  {
    SAT *sat=NULL;
/*
    if ((!(db->fp_norad=fopen(db->norad_file,"r"))))
    { printf("No file '%s'!\n",db->norad_file); return 1; }

    sat=read_msat(db->fp_norad,db->satname,NULL);
    if (!sat)
    { printf("No sat %s!\n",db->satname); return 1; }
*/
    add_to_db(db,sat,all_on);
    if (db->fp_norad) fclose(db->fp_norad); db->fp_norad=NULL;
  }

  clrmap.colors=clr;
  clrmap.size=MAXCOL;

  ld_clr(clr,SATCOL,0xff,0,0);
  ld_clr(clr,TRACKCOL,0x40,0x40,0x40);
  ld_clr(clr,RASCOL,0x7f,0x7f,0x7f);
  ld_clr(clr,RASCOL2,0xff,0xff,0xff);

  ld_clr(clr,SCANCOL,0x00,0x00,0xff);
  ld_clr(clr,SCANCOL2,0x00,0x00,0xff);

  ld_clr(clr,REFCOL,0xff,0,0xff);
  ld_clr(clr,VISCOL,0xff,0,0);

#ifdef SCR1
  width=height=0;
#endif
  main_window=Create_Window(NULL,width,height,Lab_Main,gtk_main_quit);
  menu=Create_Menu(main_window, MENU_FILE    , NULL          ,0,
                                  MENU_KF    , menu_callback ,BUTTON,
                                  MENU_Q     , gtk_main_quit ,BUTTON,
                                MENU_EDIT    , NULL          ,0,
                                  MENU_SF    , menu_callback ,BUTTON,
                                  MENU_EK    , menu_callback ,BUTTON,
                                  MENU_EDSEP ,NULL           ,0,
                                  MENU_PR    , menu_callback ,BUTTON,
                                MENU_ZOOM    , NULL          ,BUTTON,
                                  MENU_ZMFULL, menu_callback ,BUTTON,
                                  MENU_ZMIN  , menu_callback ,BUTTON,
                                  MENU_ZMOUT , menu_callback ,BUTTON,
                                  MENU_ZMIN2 , menu_callback ,BUTTON,
                                  MENU_ZMOUT2, menu_callback ,BUTTON,
                                MENU_PD      , menu_callback ,BUTTON,
                                MENU_HELP    ,NULL           ,BUTTON,
                                  MENU_PINFO ,Menu_Help      ,BUTTON,
                                0);


/* Create the canvas */
  canvas=Create_Canvas(main_window,
                cvsw,cvsh,
                configure_event_main,
                key_actions,mouse_actions,mouse_actions);

#ifdef SCR1
  canvas=Add_Scrollbars(canvas,"hbar","vbar",scroll_actions);
#endif
  {
    GtkWidget *w2,*w3,*w4;
    GtkWidget *wa,*wb,*wc,*wd;
    GtkWidget *wtijd,*wpos,*wx,*wy,*wz;
    GdkColor clr1,clr2;
    clr1.red=0xffff; clr1.green=clr1.blue=0x8888;
    clr2.red=0xffff; clr2.green=0xaaaa; clr2.blue=0xaaaa;

    w2=Create_Toggle_c(RUN,buttons_callback,FALSE,NULL,&clr2,&clr1);
    w3=Create_Toggle(ASEL,buttons_callback,FALSE);
    w4=Create_Toggle(EXT,buttons_callback,FALSE);
    wa=SPack(NULL,"v",w2,"fe5",w3,"5",w4,"5",NULL);

    wtijd=Create_ButtonArray("",time_callback,0,
                              SPIN,JAAR,"%03d%03d",1980,2040,
                              SPIN,MND,"%02d%02d",0,13,
                              SPIN,DAG,"%02d%02d",0,32,
                              SPIN,UUR,"%02d%02d",-1,24,
                              SPIN,MINT,"%02d%02d",-1,60,
                              SPIN,SEC,"%02d%02d",-1,60,
                              0
                            );
    wpos=Create_ButtonArray("",NULL,0,
                             ENTRY,LON,"%05.1f",0.,
                             ENTRY,LAT,"%05.1f",0.,
                             ENTRY,ELE,"%05.1f",0.,
                             ENTRY,AZI,"%05.1f",0.,
#define XYCALC
#ifdef XYCALC
                             ENTRY,XXX,"%05.1f",0.,
                             ENTRY,YYY,"%05.1f",0.,
#endif
                             0
                           );
    wb=Pack(NULL,'v',wtijd,5,wpos,5,NULL);

    wc=SPack(LAB_SATSEL,(db->satsel_bottom?"hs" : "vs"),NULL);
    gtk_widget_set_usize(wc,100,0);
    wd=Create_Text(LAB_TEXT,FALSE,NULL);
    gtk_widget_set_usize(wd,200,0);

    wx=Pack(NULL,'h',wa,5,wb,5,wd,5,NULL);
    if (db->satsel_bottom)
    {
      wz=Pack(NULL,'v',menu,1,canvas,1,wx,1,wc,1,NULL);
    }
    else
    {
      wy=Pack(NULL,'v',menu,1,canvas,1,wx,1,NULL);
      wz=SPack(NULL,"h",wy,"fe1",wc,"1",NULL);
    }

    gtk_container_add(GTK_CONTAINER(main_window),wz);
    update_satbut(wc,LAB_SATSEL,db->sat);
  }

  place_window(main_window,0,0,left_top);

  gtk_widget_show_all(main_window);
  gtk_widget_realize(main_window);

  Set_Enable_Update(Find_Widget(main_window,"GTK_DRAWING_AREA"),TRUE);
  configure_event_main(Find_Widget(main_window,"GTK_DRAWING_AREA"),NULL);
  refresh_wnd(main_window);

  if (db->start_now)
  {
    Set_Button(main_window,RUN,TRUE);
    buttons_callback(Find_Widget(main_window,RUN), RUN);
  }
  if (db->ext_on) Set_Button(main_window,EXT,TRUE);

  gtk_main();
  return 0;
}

void draw_points(GtkWidget *wnd,char *fn)
{
  FILE *fp;
  char l[100],*p;
  float lon, lat;
  if (!(fp=fopen(fn,"r"))) return;
  while (fgets(l,100,fp))
  {
    if (*l=='#') continue;
    if (!(p=strtok(l," "))) continue;
    lat=atof(p);
    if (!(p=strtok(NULL," "))) continue;
    lon=atof(p);
    draw_ref(wnd,D2R(lon),D2R(lat),TRUE);
  }
  fclose(fp);
}
#define TEKENPUNTENn

void refresh_wnd(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  int wnd_width;
  int wnd_height;
  wnd_width=drawing_area->allocation.width;
  wnd_height=drawing_area->allocation.height;
  draw_ref(wnd,db->refpos.lon,db->refpos.lat,TRUE);
#ifdef TEKENPUNTEN
draw_points(wnd,"scat.txt");
#endif
  Refresh_Rect(drawing_area,0,0,wnd_width,wnd_height);
}



