
#define DEF_NORADFILE "norad.txt"
#define DEF_SAT "NOAA-16"
#define DEF_MAPFILE "earthmap.gif"

#ifdef WINDHOOS
#define TYPE_COMMAND "type"
#else
#define TYPE_COMMAND "cat"
#endif

#include <float.h>
#ifndef FLT_MAX
#include <float.h>
#endif

#ifndef PI
#define PI 3.14159265
#endif

#define D2R(g) ((g)*PI/180.)     /* degree --> radians */
#define R2D(g) ((g)*180./PI)     /* radians --> degree */
#include <time.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
typedef enum {scan_unk=0,N_S,S_N,W_E,E_W} SCANDIR;

typedef enum {mode_unk=0,search_up,search_down,search_maxelev} SEARCH_MODE;

#ifndef MAX
#define MAX(a,b) ((a)>(b)? (a) : (b))
#define MIN(a,b) ((a)<(b)? (a) : (b))
#define ABS(a) ((a)<0? (-1*a) : (a))
#endif

#define USE_GTK

#ifdef USE_GTK
#include "gtk/gtk.h"
#include "gdk/gdkkeysyms.h"
#include "sgtk.h"
#else
#ifndef gint32
typedef long gint32;
#endif

#ifndef gboolean
typedef enum {FALSE=0,TRUE} gboolean;
#endif
#endif

#define PIx2 (PI*2.)           /* not present in math.h */
#define LENSATNAME 40
#define FACT 2.0

#define SEL_LAB "Sel"
#define VIS_LAB "Vis"

typedef enum { unknown_filetype,SUN,TIFF,GIF } FILE_TYPE;

#define MAX_PCICLR 4096
typedef struct clrmap
{
  int size;
  int red[MAX_PCICLR],green[MAX_PCICLR],blue[MAX_PCICLR];
} CLRMAP;

typedef struct point
{
  float x,y,z;
  float lon,lat;
} POINT;

typedef struct direction
{
  float elev,azim;
  float x,y;
} DIRECTION;


typedef struct kepler
{
  int    epoch_year;
  float  epoch_day;
  float  decay_rate;
  float  inclination;
  float  raan;
  float  eccentricity;
  float  perigee;
  float  anomaly;
  float  motion;
  gint32 epoch_rev;
} KEPLER;

typedef struct orbit
{
  gboolean valid;                  /* flag: kepler data valid? */

/* Start time record (used) */
  struct tm start_tm;              /* record start time  */
  int       start_ms;              /* record start time msec */

/* Start time record (satellite) */
  struct tm start_tm_sat;          /* record start time  */
  int       start_ms_sat;          /* record start time msec */

/* Start time record (computer) */
  struct tm start_tm_comp;          /* record start time  */

/* Reference time Kepler data */
  gint32    ref_time;              /* total reference time  */
  struct tm ref_tm;                /* reference time year ... sec */
  int       ref_ms;                /* reference time msec */

  float height;                    /* sat height [meters] */ 
  float loop_time;                 /* time 1 loop [seconds] */
  float k2_n;
  float k2_w;
  float dh_w;

/* Some satellite dependent constants */
  float max_sens_angle;
  SCANDIR scan_hor,scan_ver;       /* Movement of satellite */
  int width_original;

  gboolean use_sattime;
  int offset_gmt;
  int offset_sec;
  float offset_lon;
} ORBIT;

typedef struct sat
{
  struct sat *prev,*next;
  struct dbase *db;
  char satname[40];
  KEPLER kepler;
  ORBIT orbit;
  POINT pos;
  DIRECTION dir;
  gboolean visible; 
  gboolean selected; 
} SAT;

typedef struct dbase
{
  POINT refpos;
  SAT *sat;
  SAT *sat_sel;
  SAT *next_sat;

  struct tm glob_tm;
  struct tm genstart_tm;
  struct tm genrange_tm;

  char satname[40];
  char norad_file[100];
  char pref_noradfile[100];
  char map_file[100];
  char pref_mapfile[100];
  FILE *fp_norad;
/*
  int hour_offset;
*/
  float time_offset;
  char ofile[50];
  float ppl;             /* zoom */
  float fact;             /* zoom */
  float zx,zy;
  int ox,oy;
  gboolean redraw;
  gboolean redrawing;
  gboolean setscrolflag;
  float elev_hor;
  struct track *track;
  gboolean start_now;
  gboolean ext_on;
  gboolean satsel_bottom;
  gboolean auto_select;
  gboolean is_gmtime;

  char prog_up[100];
  char prog_down[100];
  char prog_track[100];
  char prog_trackup[100];

  int ps_x_offset,ps_y_offset;
  int ps_max_h;
  int ps_max_t;
  int reptime;

  char used_preffile[300];         /* preffile used to read from */

  char *cur_dir;
  char *home_dir;
  char *prog_dir;

  int fontsize;
} DBASE;

typedef struct track
{
  struct track *prev,*next;
  POINT up_pos;
  struct tm up_time;

  POINT down_pos;
  struct tm down_time;

  POINT maxelev_pos;
  struct tm maxelev_time;
  float max_elev;

  gboolean going_south;
} TRACK;




typedef struct picinfo
{
  char file_name[100];             /* name of current file */
  FILE *fp;                        /* filepointer; NULL if no file opened */
  char sat_name[LENSATNAME];       /* name satellite */

  int width,height,depth;          /* Size picture */
  float alpha_max;                 /* satellite sensor angle */
  long pic_strt;                   /* pic starts at this byte number */

  int width_original;              /* Size original pic */
  int ox,oy;                       /* Offset to original pic */
  int lines_per_sec;               /* lines per second */

  SCANDIR scan_hor,scan_ver;       /* Movement of satellite */
  struct tm picture_time;          /* Time of pic (not always time of record) */

  KEPLER kepler;                   /* Keplerian elements of sat orbit */
  ORBIT orbit;                     /* Orbit vars */
} PICINFO;

#define Rewind(str) while ((str) && (str->prev)) str=str->prev

void yday2mday_mon(struct tm *tmref);
gint32 mktime_ntz(struct tm *tm);
float alpha_2_beta(float ,SCANDIR,int ,int ,float );
void calceleazim(POINT *,ORBIT *,POINT *,DIRECTION *);
void Read_Prefs(char *,DBASE *);
void Save_Prefs(DBASE *,gboolean);
void Create_preferences_wnd(GtkWidget *);


